﻿using System;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.Threading;
using Antiplagiat.ApiCorp.Model;

namespace ApiCorpExample
{
    class ExamplesNative
    {
        /// <summary>
        /// Url of "Antiplagiat" website.
        /// </summary>
        public static Uri AntiplagiatUri { get; private set; }

        /// <summary>
        /// Unique company name.
        /// </summary>
        public static string CompanyName { get; private set; }

        /// <summary>
        /// Apicorp service url.
        /// </summary>
        public static string ApiCorpAddress { get; private set; }

        // The system remembers the login/ password that was used at the first request. The login/password specified at the first request is used for further verification of requests.
        public static string Login { get; private set; }
        public static string Password { get; private set; }

        static ExamplesNative()
        {
            AntiplagiatUri = new Uri(ConfigurationManager.AppSettings["antipalgiat.url"]);
            CompanyName = ConfigurationManager.AppSettings["company"];
            ApiCorpAddress = ConfigurationManager.AppSettings["apicorp.addr"];
            Login = ConfigurationManager.AppSettings["login"];
            Password = ConfigurationManager.AppSettings["password"];
        }

        /// <summary>
        /// Preparation of the description of the uploaded file.
        /// </summary>
        /// <param name="fileName">Full path to the file. </param>
        /// <returns>A prepared object with a description and the uploaded file itself.</returns>
        private static DocData GetDocData(string fileName)
        {
            return new DocData
            {
                Data = File.ReadAllBytes(fileName),
                FileType = Path.GetExtension(fileName),
                FileName = Path.GetFileName(fileName),
                ExternalUserID = "ivanov", //the field must be filled in with the ID of the real user in the external system
            };
        }

        /// <summary>
        /// An example of the most common scenario of the document check.
        /// </summary>
        /// <param name="fileName">The path to the file to be checked.</param>
        public static void SimpleCheck(string fileName)
        {
            Console.WriteLine("SimpleCheck filename=" + fileName);
            // Create a service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress);

            // Description of the uploaded file
            var data = GetDocData(fileName);
            var attributes = new DocAttributes
            {
                DocumentDescription = new VerificationReportOptions()
                {
                    Authors = new[]
                    {
                        new AuthorName
                        {
                            OtherNames = "Иван Иванович",
                            Surname = "Иванов",
                            PersonIDs = new PersonIDs { CustomID = "original" }
                        }
                    }
                }
            };

            // Document upload
            var uploadResult = client.UploadDocument(data, attributes);

            // Document ID. If the archive is not being uploaded, then the list of downloaded documents will consist of one element.
            var id = uploadResult.Uploaded.First().Id;

            // Send to check using all available searching modules 
            client.CheckDocument(id);
            // Send to check using only own collection and "wikipedia" searching module. To get all searchilg modules list use example GetTariffInfo()
            //client.CheckDocument(id, new[] { CompanyName, "checker" });

            // Get status of the last check
            var status = client.GetCheckStatus(id);

            // The cycle of waiting for the end of the check
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(id);
            }

            // Check failed
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("An error occurred while checking the document {0}: {1}", fileName, status.FailDetails);
                return;
            }

            // Get brief report
            var report = client.GetReportView(id);
            //Total percentage of reuses found
            Console.WriteLine("Report Summary: " + (100 - report.Summary.DetailedScore.Unknown) + "%");
            // Details of the percentage of non-original text divided into reuses, legal citation and self-citation
            Console.WriteLine("Report BaseScore.Legal: " + report.Summary.BaseScore.Legal + "%");
            Console.WriteLine("Report BaseScore.Plagiarism: " + report.Summary.BaseScore.Plagiarism + "%");
            Console.WriteLine("Report BaseScore.SelfCite: " + report.Summary.BaseScore.SelfCite + "%");
            // Details of the percentage of non-original text divided into reuses, legal citation and self-citation after correction
            Console.WriteLine("Report DetailedScore.Legal: " + report.Summary.DetailedScore.Legal + "%");
            Console.WriteLine("Report DetailedScore.Plagiarism: " + report.Summary.DetailedScore.Plagiarism + "%");
            Console.WriteLine("Report DetailedScore.SelfCite: " + report.Summary.DetailedScore.SelfCite + "%");

            foreach (var checkService in report.CheckServiceResults)
            {
                // Information about each search module 
                Console.WriteLine("Check service: {0} ({1}) \n ScoreByReport={2}% \n Score.White={3}% Score.Black={4}% Score.SelfCite={5}% \n"
                    , checkService.CheckServiceName
                    , checkService.CollectionDescription
                    , checkService.ScoreByReport
                    , checkService.ScoreByReport.Legal
                    , checkService.ScoreByReport.Plagiarism
                    , checkService.ScoreByReport.SelfCite
                    );
                if (checkService.Sources == null) continue;
                foreach (var source in checkService.Sources)
                    // Information about each found source
                    Console.WriteLine("\t{0}: Score={1}%({2}%), Name='{3}' Author='{4}' Url='{5}'",
                        source.SrcHash,
                        source.ScoreByReport, source.ScoreBySource,
                        source.Name, source.Author, source.Url);
            }

            // Get full report
            var fullReport = client.GetReportView(id, new ReportViewOptions { FullReport = true, 
                NeedText = true, NeedAttributes = true});

            if (fullReport.Details.CiteBlocks != null)
            {
                // Find the biggest block of reuses and display it 
                CiteBlock maxBlock = null;
                foreach (var block in fullReport.Details.CiteBlocks)
                {
                    if ((maxBlock == null) || block.Length > maxBlock.Length)
                    {
                        maxBlock = block;
                    }
                }
                if (maxBlock != null)
                    Console.WriteLine("Max block length={0} Source={1} text:\n{2}...", maxBlock.Length, maxBlock.SrcHash,
                        fullReport.Details.Text.Substring(maxBlock.Offset, Math.Min(maxBlock.Length, 200)));
            }

            var authorsData = fullReport.Attributes.DocumentDescription.Authors;
            Console.WriteLine("Author Surname={0} OtherNames={1} CustomId={2}",
                authorsData[0].Surname, authorsData[0].OtherNames, authorsData[0].PersonIDs.CustomID);
        }

        /// <summary>
        /// An example demonstrating how to behave correctly in case of errors during the loading of a document to check.
        /// </summary>
        /// <param name="fileName">Path to the file which should be checked.</param>
		public static void IncorrectUpload(string fileName)
        {
            Console.WriteLine("IncorrectUpload filename=" + fileName);
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Description of the uploaded file
            var data = GetDocData(fileName);

            // File uploading
            try
            {
                var uploadResult = client.UploadDocument(data);
                Console.WriteLine("File is uploaded successfully");
            }
            catch (FaultException<InvalidArgumentException> e)
            {
                // This file cannot be downloaded. The specific reason is indicated in the field Message.
                // Repeating the call with the same parameters is pointless, the result will be the same.
                Console.WriteLine("File can't be uploaded: " + e.Message);
            }
            catch (FaultException e)
            {
                // The problem is not related to a specific file, it makes sense to try again later
                Console.WriteLine("An error occurred while uploading the file: " + e.Message);
            }
        }


        /// <summary>
        /// Enumeration of all received reports
        /// </summary>
        public static void EnumerateReports()
        {
            Console.WriteLine("EnumerateReports");
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();
            // Selecting user documents with an ID in the external system "ivanov"
            var opts = new EnumerateReportsOptions { ExternalUserID = "ivanov", Count = 10 };
            // Get the first pack of reports, sorting is always reverse chronological, the report of the last uploaded document will be the first
            var reps = client.EnumerateReportInfos(opts);
            // Report counter
            var i = 0;
            // The EnumerateReportInfos function returns the results of the check in pack. An empty pack means that the search has ended.
            while (reps.Length > 0)
            {
                foreach (var info in reps)
                {
                    ++i;
                    // We output the number in order, the name of the document and the status of the last report
                    Console.WriteLine("{0}\tDocName: {1}\n   Status: {2}", i, info.DocumentInfo.Attributes.Name, info.CheckStatus.Status);
                    // We output the number in order, the name of the document and the status of the last report
                    if (info.CheckStatus.Status == ReportStatus.Ready)
                        Console.WriteLine("{0}, {1}", 100 - info.CheckStatus.Summary.DetailedScore.Unknown, new Uri(AntiplagiatUri, info.CheckStatus.Summary.ReportWebId));
                }
                // Get the next pack of reports
                opts.Skip = i;
                reps = client.EnumerateReportInfos(opts);
            }
        }


        /// <summary>
        /// Enumeration of all uploaded documents
        /// </summary>
        public static void EnumerateDocuments()
        {
            Console.WriteLine("EnumerateDocuments");
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();
            // Get the first pack of documents
            var docs = client.EnumerateDocuments(null);
            // Document counter
            var i = 0;
            // The EnumerateDocuments function returns IDs in packs. An empty pack means that the search has ended.
            while (docs.Length > 0)
            {
                foreach (var id in docs)
                {
                    ++i;
                    // We add every fifth document to the index
                    if ((i % 5) == 0)
                    {
                        client.SetIndexState(id, IndexState.Indexed);
                    }

                    // We add an attribute to every third document
                    if ((i % 3) == 0)
                    {
                        var docnum = new CustomAttribute { AttrName = "docnum", AttrValue = "" + i };
                        client.UpdateDocumentAttributes(id, new DocAttributes { Custom = new[] { docnum } });
                    }
                }
                // Get the next pack of document IDs
                docs = client.EnumerateDocuments(docs[docs.Length - 1]);
            }
            Console.WriteLine("Total documents count: {0} ({1})", i, client.GetCompanyStats().TotalCount);
        }

        /// <summary>
        /// Enumeration of all documents added to the index
        /// </summary>
        public static void EnumerateIndex()
        {
            Console.WriteLine("EnumerateIndex");
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // When calling EnumerateDocuments, the options indicate that only documents added to the index are required
            var enumerateOptions = new EnumerateDocumentsOptions { AddedToIndex = true };
            var docs = client.EnumerateDocuments(null, enumerateOptions);
            // Document counter
            var i = 0;
            // The EnumerateDocuments function returns IDs in packs. An empty pack means that the enumeration has ended.
            while (docs.Length > 0)
            {
                foreach (var id in docs)
                {
                    ++i;
                    // We delete every second document from the index
                    if ((i % 2) == 0)
                    {
                        client.SetIndexState(id, IndexState.None);
                    }
                }
                // Get next pack of document IDs
                docs = client.EnumerateDocuments(docs[docs.Length - 1], enumerateOptions);
            }
            Console.WriteLine("Total indexed documents count: {0} ({1})", i, client.GetCompanyStats().AddedToIndexCount);
        }

        /// <summary>
        /// Check the document, get various links to the report on the "Antiplagiat" website
        /// </summary>
        /// <param name="fileName">The path to the file to be checked.</param>
        public static void GetWebReport(string fileName)
        {
            Console.WriteLine("GetWebReport filename=" + fileName);
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Description of the uploaded file
            var data = GetDocData(fileName);

            // File upload
            var uploadResult = client.UploadDocument(data);

            // Document ID. If the archive is not being uploaded, then the list of uploaded documents will consist of one element.
            DocumentId id = uploadResult.Uploaded.First().Id;

            // Send to check
            Console.WriteLine(id.ToString());
            client.CheckDocument(id);

            // Get the current status of the last check
            var status = client.GetCheckStatus(id);

            // The cycle of waiting for the end of the check
            while (ReportStatus.InProgress == status.Status)
            {
                var estTime = TimeSpan.FromSeconds(status.EstimatedWaitTime.Value);
                Console.WriteLine(estTime);
                Thread.Sleep(estTime);
                status = client.GetCheckStatus(id);
            }

            // Check failed
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("При проверке документа {0} произошла ошибка: {1}", fileName, status.FailDetails);
                return;
            }

            // Get link to the full report on the "Antiplagiat" site
            var webFullUrl = new Uri(AntiplagiatUri, status.Summary.ReportWebId);
            Console.WriteLine("Full Report: " + webFullUrl);

            // Get link to the brief report on the "Antiplagiat" site
            var webShortUrl = new Uri(AntiplagiatUri, status.Summary.ShortReportWebId);
            Console.WriteLine("Short Report: " + webShortUrl);

            // Get link to the read-only report on the "Antiplagiat" site
            var webReadonlyUrl = new Uri(AntiplagiatUri, status.Summary.ReadonlyReportWebId);
            Console.WriteLine("Readonly Report: " + webReadonlyUrl);
        }

        /// <summary>
        /// An example demonstrating methods for working with folders.
        /// </summary>
        /// <param name="fileName1">An example demonstrating methods for working with folders.</param>
        /// <param name="fileName2">The path to the second file that will be uploaded to the system.</param>
        public static void FoldersMethods(string fileName1, string fileName2)
        {
            Console.WriteLine("FoldersMethods");
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            const string email = "testapi@antiplagiat.ru";

            // User creates folder "New Folder"
            var folderId = client.AddFolder(email, "New folder");

            // User renames it to "Documents"
            client.RenameFolder(email, folderId, "Documents");

            // Folder creation options.
            var folderOptions = new FolderOptions { ParentId = folderId };

            // User creates in the folder "Documents" a new folder "Important"
            var importantFolderId = client.AddFolder(email, "Important", folderOptions);

            // Description of the first uploaded file
            var data1 = GetDocData(fileName1);

            // User uploads a file fileName1 in the folder "Documents"
            client.UploadDocument(data1, null, new UploadOptions { FolderId = folderId, FromUser = email });

            // Description of the second uploaded file
            var data2 = GetDocData(fileName2);

            // The user uploads the file fileName2 to the folder "Important"
            client.UploadDocument(data2, null, new UploadOptions { FolderId = importantFolderId, FromUser = email });

            // User creates folder "Trash"
            var trashFolderId = client.AddFolder(email, "Trash");

            // The user gets the IDs of all his documents
            var docIds = client.EnumerateDocuments(null, new EnumerateDocumentsOptions { FromUser = email });

            // The user moves his documents to the folder "Trash"
            client.MoveDocuments(email, trashFolderId, docIds);

            // The user gets a folder tree
            var folderTree = client.GetFolders(email);

            Console.WriteLine("FolderTree:");

            Console.WriteLine("\nTop level:");
            Console.WriteLine(string.Format("Name: {0}", folderTree.Name));

            Console.WriteLine("\nNext level:");

            foreach (var folder in folderTree.Children)
                Console.WriteLine(string.Format("Name: {0}", folder.Name));

            Console.WriteLine("\nGet documents from \"Trash\"");

            // User gets his documents from the folder "Trash"
            var trashDocs = client.EnumerateDocuments(null, new EnumerateDocumentsOptions { InStorage = false, FolderId = trashFolderId, FromUser = email });

            Console.WriteLine("Documents count: {0}", trashDocs.Length);

            // User deletes folders "Documents" and "Trash"
            client.DeleteFolder(email, folderId);
            client.DeleteFolder(email, trashFolderId);
        }

        /// <summary>
        /// Methods for working with the storage.
        /// </summary>
        /// <param name="fileName1">The path to the first file that will be uploaded to the system.</param>
        /// <param name="fileName2">The path to the second file that will be uploaded to the system.</param>
        public static void StorageMethods(string fileName1, string fileName2)
        {
            Console.WriteLine("StorageMethods");
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // User creates folder "New Folder" in the storage
            var folderId = client.AddFolder(null, "New folder");

            // User renames it into "Documents"
            client.RenameFolder(null, folderId, "Documents");

            // Folder creation options
            var folderOptions = new FolderOptions { ParentId = folderId };

            // User creates new folder "Important" in the folder "Documents"
            var importantFolderId = client.AddFolder(null, "Important", folderOptions);

            // Description of the first uploaded file
            var data1 = GetDocData(fileName1);

            // The user uploads file fileName1 to the folder "Documents"
            client.UploadDocument(data1, null, new UploadOptions { FolderId = folderId, ToStorage = true });

            // Description of the second uploaded file
            var data2 = GetDocData(fileName2);

            // The user uploads file fileName2 to the folder "Documents"
            client.UploadDocument(data2, null, new UploadOptions { FolderId = importantFolderId, ToStorage = true });

            // User creates folder "Trash" in the storage
            var trashFolderId = client.AddFolder(null, "Trash");

            // The user gets the IDs of all documents from the storage
            var docIds = client.EnumerateDocuments(null);

            // The user moves all documents to the folder "Trash"
            client.MoveDocuments(null, trashFolderId, docIds);

            // The user gets a tree of storage folders
            var folderTree = client.GetFolders(null);

            Console.WriteLine("FolderTree:");

            Console.WriteLine("\nTop level:");
            Console.WriteLine(string.Format("Name: {0}", folderTree.Name));

            Console.WriteLine("\nNext level:");

            foreach (var folder in folderTree.Children)
                Console.WriteLine(string.Format("Name: {0}", folder.Name));

            Console.WriteLine("\nGet documents from \"Trash\"");

            // User gets the documents fro the folder "Trash" in the storage
            var trashDocs = client.EnumerateDocuments(null, new EnumerateDocumentsOptions { InStorage = true, FolderId = trashFolderId });

            Console.WriteLine("Documents count: {0}", trashDocs.Length);

            // User deletes folders "Documents" and "Trash"
            client.DeleteFolder(null, folderId);
            client.DeleteFolder(null, trashFolderId);
        }

        /// <summary>
        /// Demonstration of methods for getting information about the current tariff.
        /// </summary>
        public static void GetTariffInfo()
        {
            Console.WriteLine("GetTariffInfo");

            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Get information about the current tariff
            var tariffInfo = client.GetTariffInfo();

            Console.WriteLine(string.Format("Tariff name: {0}", tariffInfo.Name));
            Console.WriteLine(string.Format("Date of tariff start: {0}", tariffInfo.SubscriptionDate));
            Console.WriteLine(string.Format("Date of tariff end: {0}", tariffInfo.ExpirationDate));
            Console.WriteLine(string.Format("Initial number of checks: {0}", tariffInfo.TotalChecksCount));
            Console.WriteLine(string.Format("Remaining number of checks: {0}", tariffInfo.RemainedChecksCount));

            Console.WriteLine(string.Format("\nList of check services by which the document can be checked:"));

            foreach (var checkServiceInfo in tariffInfo.CheckServices)
                Console.WriteLine(string.Format("{0} ({1})", checkServiceInfo.Code, checkServiceInfo.Description));
        }

        /// <summary>
        /// An example demonstrating the receipt of a pdf report.
        /// </summary>
        /// <param name="fileName">The path to the file to be checked.</param>
        public static void ExportReportToPdf(string fileName)
        {
            Console.WriteLine("ExportReportToPdf filename=" + fileName);
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Description of the uploaded file
            var data = GetDocData(fileName);

            //File uploading
            var uploadResult = client.UploadDocument(data);

            // Document ID. If the archive is not being downloaded, then the list of downloaded documents will consist of one element
            DocumentId id = uploadResult.Uploaded.First().Id;

            // Send ro check
            client.CheckDocument(id);

            // Get status of the last check
            var status = client.GetCheckStatus(id);

            // The cycle of waiting for the end of the check
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(id);
            }

            // Check failed
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("An error occurred while checking the document {0}: {1}", fileName, status.FailDetails);
                return;
            }

            // Request the formation of the last full report in PDF format.
            var exportReportInfo = client.ExportReportToPdf(id);

            // Waiting cycle for report generation
            while (ExportReportStatus.InProgress == exportReportInfo.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(exportReportInfo.EstimatedWaitTime.Value));
                exportReportInfo = client.ExportReportToPdf(id);
            }

            // Report generation failed
            if (ExportReportStatus.Failed == exportReportInfo.Status)
            {
                Trace.TraceInformation("An error occurred when generating a PDF report for document {0}.", fileName);
                return;
            }

            // Get a link to the report on the "Antiplagiat" website
            // Please pay attention that it is not guaranteed that this link will work forever, it may stop working at any time,
            // therefore, you cannot give it to the user. You need to download a pdf for yourself and then manage its lifetime
            var downloadLink = new Uri(AntiplagiatUri, exportReportInfo.DownloadLink);
            Console.WriteLine($"PDF full report (number = {exportReportInfo.ReportNum}): " + downloadLink);

            // Options for generating a brief report with the number 1 in PDF format.
            var options = new ExportReportOptions
            {
                ReportNum = 1,
                ShortReport = true
            };

            // Request the generation of a PDF report with the specified options.
            exportReportInfo = client.ExportReportToPdf(id, options);

            // The cycle of waiting for the report generation
            while (ExportReportStatus.InProgress == exportReportInfo.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(exportReportInfo.EstimatedWaitTime.Value));
                exportReportInfo = client.ExportReportToPdf(id, options);
            }

            // Report generation failed.
            if (ExportReportStatus.Failed == exportReportInfo.Status)
            {
                Trace.TraceInformation("При формировании PDF-отчета для документа {0} произошла ошибка.", fileName);
                return;
            }

            // Get a link to the report on the "Antiplagiat" website
            // Please pay attention that it is not guaranteed that this link will work forever, it may stop working at any time,
            // therefore, you cannot give it to the user. You need to download a pdf for yourself and then manage its lifetime
            downloadLink = new Uri(AntiplagiatUri, exportReportInfo.DownloadLink);
            Console.WriteLine($"PDF short report (number = {exportReportInfo.ReportNum}): " + downloadLink);
        }

        /// <summary>
        /// An example of getting a certificate for the checked file
        /// </summary>
        /// <param name="fileName">Путь до файла который будет проверяться</param>
        public static void GetVerificationReport(string fileName)
        {
            Console.WriteLine("GetVerificationReport fileName={0}", fileName);

            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress, Login, Password).UseMtomMessageEnconding();

            // Description of the file being uploaded
            var data = GetDocData(fileName);

            // File uploading
            var uploadResult = client.UploadDocument(data);

            // Document ID. If the archive is not being uploaded, then the list of downloaded documents will consist of one element.
            DocumentId id = uploadResult.Uploaded.First().Id;

            // Send to check
            Console.WriteLine("Checking the document: {0}", id.ToString());
            client.CheckDocument(id);

            // Get current status of the last check
            var status = client.GetCheckStatus(id);

            // The cycle of waiting for the end of the check
            while (ReportStatus.InProgress == status.Status)
            {
                var estTime = TimeSpan.FromSeconds(status.EstimatedWaitTime.Value);
                Console.WriteLine("Estimated processing time: {0}", estTime);
                Thread.Sleep(estTime);
                status = client.GetCheckStatus(id);
            }

            // Check failed.
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("while checking the document {0} an error occurred : {1}", fileName, status.FailDetails);
                return;
            }

            try
            {
                //You can download certificate with or without filled fields
                //Without filled fields
                //var reportBytes = client.GetVerificationReport(id);

                //To fill the fields, you need to create a certificate options object with the desired property values 
                var options = new VerificationReportOptions
                {
                    Author = "Ivanov I.I.", // Full name of the document's author
                    Department = "Faculty of General and Applied Physics", // Faculty (structural subdivision)
                    ShortReport = true, // Is a link to the summary brief required? (qr code)
                    Type = "Thesis", // Type of work
                    Verifier = "Petrov P.P.", // Full name of the verifier
                    Work = "Coherent spin dynamics of a two-dimensional electron gas"  // Name of the work
                };

                var reportPath = getVerReportName(id.Id, options);

                //With filled fields
                var reportBytesWithFields = client.GetVerificationReport(id, options);

                //writing to file
                File.WriteAllBytes(reportPath, reportBytesWithFields);
            }
            catch (FaultException<InvalidArgumentException> e)
            {
                // The document does not have a report/closed report, or null was passed as the id in GetVerificationReport
                Console.WriteLine("A certificate for the document can't be received: " + e.Message);
            }
            catch (FaultException<DocumentIdException> e)
            {
                // Invalid DocumentId is specified, when passed null sends FaultException<InvalidArgumentException>
                Console.WriteLine("Certificate for the document could not be received due to an incorrectly passed documentId: " + e.Message);
            }
            catch (FaultException e)
            {
                // The problem is not related to the parameters being passed, it makes sense to try again later
                Console.WriteLine("Справка для документа не может быть получена: " + e.Message);
            }
        }

        /// <summary>
        /// Checking the calculation of self-citation for its own collection.
        /// </summary>
        /// <param name="fileName">The path to the file to be checked.</param>
        public static void SelfCiteCheck(string fileName)
        {
            Console.WriteLine("SelfCite filename=" + fileName);
            // Create service client
            var client = new Antiplagiat.ApiCorp.ApiCorpClient(CompanyName, ApiCorpAddress);

            // Description of the file being uploaded
            var data = GetDocData(fileName);
            var attributes = new DocAttributes
            {
                DocumentDescription = new VerificationReportOptions()
                {
                    Authors = new[]
                    {
                        new AuthorName
                        {
                            OtherNames = "Ivan",
                            Surname = "Ivanov",
                            PersonIDs = new PersonIDs { CustomID = "original" }
                        }
                    }
                }
            };
            var options = new UploadOptions
            {
                AddToIndex = true
            };

            // File uploading
            var uploadResult = client.UploadDocument(data, attributes, options);

            // Document ID. If the archive is not being uploaded, then the list of downloaded documents will consist of one element.
            var id = uploadResult.Uploaded.First().Id;

            // Send to check using all available searching modules
            client.CheckDocument(id, new[] { CompanyName });

            // Get current status of the last check
            var status = client.GetCheckStatus(id);

            // The cycle of waiting for the end of the check
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(id);
            }

            // Check failed
            if (ReportStatus.Failed == status.Status)
            {
                Trace.TraceInformation("While checking the document {0} an error occurred: {1}", fileName, status.FailDetails);
                return;
            }

            //we upload the same file with the same author Id to check the calculation of self-citation
            uploadResult = client.UploadDocument(data, attributes);
            var idSelfCite = uploadResult.Uploaded.First().Id;

            client.CheckDocument(idSelfCite, new[] { CompanyName });

            // Get current status of the last check
            status = client.GetCheckStatus(idSelfCite);

            // The cycle of waiting for the end of the check
            while (ReportStatus.InProgress == status.Status)
            {
                Thread.Sleep(TimeSpan.FromSeconds(status.EstimatedWaitTime.Value));
                status = client.GetCheckStatus(idSelfCite);
            }

            // Get brief report
            var report = client.GetReportView(idSelfCite);
            // There should be a 100% self-citation
            Console.WriteLine("Report BaseScore.SelfCite: " + report.Summary.BaseScore.SelfCite + "%");
        }

        private static string getVerReportName(int? id, VerificationReportOptions verReportOptions)
        {
            var docId = id.HasValue ? id.Value + "_" : "";
            string author = "";

            if (verReportOptions != null)
                if (!string.IsNullOrEmpty(verReportOptions.Author))
                    author = "_" + verReportOptions.Author;

            return $"Сertificate_{docId}{DateTime.Now.ToString("yyyyMMdd")}{author}.pdf";
        }
    }
}